! $Id: ContainsStmts.f90 981 2013-03-12 10:36:29Z ian $
! ff08 source code copyright 2012 M.E.G.M.S.  See LICENCE.txt for licence.
!> @file
!! Defines the ContainsStmts module.


!*******************************************************************************
!!
!> Procedure for parsing a /contains-stmt/ (R1242).

MODULE ContainsStmts
  
  IMPLICIT NONE
  
  PRIVATE
  
  !-----------------------------------------------------------------------------
  
  PUBLIC :: ParseContainsStmt
  
CONTAINS
  
  !*****************************************************************************
  !!
  !> Parse a sequence of tokens that is a /contains-stmt/ (R1242).
  !!
  !! @param[in]     tlist             The sequence of tokens.
  !!
  !! @param[out]    err_list          List of errors.
  
  SUBROUTINE ParseContainsStmt(tlist, err_list)
    
    USE CompilerKinds
    USE Errors
    USE ErrorCodes
    USE ParseUtils
    USE Tokens
    
    !---------------------------------------------------------------------------
    ! Arguments
    
    TYPE(Token), INTENT(IN) :: tlist(:)
    TYPE(Error), INTENT(OUT), ALLOCATABLE :: err_list(:)
    
    !---------------------------------------------------------------------------
    ! Local constants
    
    ! Syntax rule number for a /contains-stmt/.
    CHARACTER(*), PARAMETER :: comp = 'R1242'
    
    !---------------------------------------------------------------------------
    ! Local variables
    
    TYPE(Error) :: err        ! Error object for Expect calls.
    
    !***************************************************************************
    
    ALLOCATE(err_list(0))
    
    IF (SIZE(tlist) == 0) THEN
      CALL Add( err_list, CODE=errSyntax, COMPONENT=comp,  &
          MSG='Expected /contains-stmt/.' )
      RETURN
    END IF
    
    !---------------------------------------------------------------------------
    
    ! CONTAINS ...
    CALL ExpectKeyword(tlist, 1, scck_'CONTAINS', err)
    CALL Add(err_list, err, COMPONENT=comp)
    IF (Failure(err)) RETURN
    
    ! Nothing else!
    CALL ExpectEOS(tlist, 2, comp, err_list)
    
  END SUBROUTINE ParseContainsStmt
  
END MODULE ContainsStmts
